@@ -22,7 +22,7 @@ from .mesonlib import default_libdir, default_libexecdir, default_prefix
 import ast
 import argparse
 
-version = '0.47.0'
+version = '0.47.1'
 backendlist = ['ninja', 'vs', 'vs2010', 'vs2015', 'vs2017', 'xcode']
 
 default_yielding = False
@@ -169,7 +169,7 @@ class UserArrayOption(UserOption):
         if len(set(newvalue)) != len(newvalue):
             msg = 'Duplicated values in array option "%s" is deprecated. ' \
                   'This will become a hard error in the future.' % (self.name)
-            mlog.log(mlog.red('DEPRECATION:'), msg)
+            mlog.deprecation(msg)
         for i in newvalue:
             if not isinstance(i, str):
                 raise MesonException('String array element "{0}" is not a string.'.format(str(newvalue)))
